/* Copyright (C) 2005 Juergen "George" Sawinski
 * All rights reserved.
 * Licensed under the LGPL v2.1 or later,
 * see accompanying COPYING.LIB file.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <lauxlib.h>
#include <stdlib.h>
#include <libtar.h>

#include "tar.h"

/*****************************************************************************
 * Relate file-desc to compression stream
 *****************************************************************************/
struct streams {
  struct streams *next;

  int fd;
  void *F;
};

static struct streams streams = { NULL, 0, NULL };

void  luatar_cstream_add(int fd, void *F)
{
  struct streams *ptr = &streams;
  struct streams *stream = NULL;

  dbg("_stream_add(%d,%p)\n", fd, F);

  /* get last element */
  while (ptr->next) ptr = ptr->next;

  /* create node */
  stream = (struct streams *) malloc(sizeof(struct streams));
  stream->next = NULL;
  stream->fd = fd;
  stream->F = F;

  /* append */
  ptr->next = stream;
}

void  _stream_remove(int fd)
{
  struct streams *ptr = &streams;
  struct streams *stream = NULL;

  dbg("_stream_remove(%d)\n", fd);

  /* find node */
  while (ptr->next && (ptr->next->fd != fd)) ptr = ptr->next;
  stream = ptr->next;

  if (stream == NULL) {
    fprintf(stderr, "Internal error in luatar: list node empty\n");
    abort();
  }

  /* reconnect list */
  ptr->next = stream->next;

  /* delete */
  free(stream);
}

void *_stream_get(int fd)
{
  struct streams *ptr = &streams;

  /* find node */
  while (ptr && (ptr->fd != fd)) ptr = ptr->next;

  /* report stream */
  if (ptr) {
    dbg("_stream_get(%d) -> %p\n", fd, ptr->F);
    return ptr->F;
  }

  dbg("_stream_get(%d) -> nil", fd);
  return NULL;
}

/*****************************************************************************
 * Compression methods
 *****************************************************************************/
#ifdef USE_GZIP
#include <zlib.h>

static int gzip_closefunc(int fd)
{
    void *F = _stream_get(fd);

    gzclose(F);
    _stream_remove(fd);

    return 0;
}

static ssize_t gzip_readfunc(int fd, void *buf, size_t count)
{
    void *F = _stream_get(fd);

    return gzread(F, buf, count);
}

static ssize_t gzip_writefunc(int fd, const void *buf, size_t count)
{
    void *F = _stream_get(fd);

    return gzwrite(F, buf, count);
}

tartype_t c_gzip = { NULL, gzip_closefunc, gzip_readfunc, gzip_writefunc };
#else
tartype_t c_gzip = { NULL, NULL, NULL, NULL };
#endif

#ifdef USE_BZIP2
#include <bzlib.h>

static int bzip2_closefunc(int fd)
{
    void *F = _stream_get(fd);

    BZ2_bzclose(F);
    _stream_remove(fd);

    return 0;
}

static ssize_t bzip2_readfunc(int fd, void *buf, size_t count)
{
    void *F = _stream_get(fd);
    return BZ2_bzread(F, buf, count);
}

static ssize_t bzip2_writefunc(int fd, const void *buf, size_t count)
{
    void *F = _stream_get(fd);
    return BZ2_bzwrite(F, (void *) buf, count);
}

tartype_t c_bzip2 = { NULL, bzip2_closefunc, bzip2_readfunc, bzip2_writefunc };
#else
tartype_t c_bzip2 = { NULL, NULL, NULL, NULL };
#endif

